#define PROG_NAME "dm_test_005_coding"
#define PROG_DESC "test of DNA signal encoding/decoding routines"
#define PROG_VERS "1.0"

/* Last edited on 2008-01-30 12:33:15 by hcgl */

#define dm_test_005_coding_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#define PROG_HELP \
  PROG_NAME " {NSAMPLES} {OUT_NAME}"

#define PROG_INFO \
  "NAME\n" \
  "  " PROG_NAME " - " PROG_DESC "\n" \
  "\n" \
  "SYNOPSIS\n" \
  "  " PROG_HELP "\n" \
  "\n" \
  "DESCRIPTION\n" \
  "  This program checks the signal encoding and decoding" \
  " routines with {NSAMPLES} samples.\n" \
  "\n" \
  "  All output files will have names starting with {OUT_NAME}.\n" \
  "\n" \
  "OPTIONS\n" \
  "  None.\n" \
  argparser_help_info_HELP_INFO "\n" \
  "SEE ALSO\n" \
  "  dm_test_filter(1)\n" \
  "\n" \
  "AUTHOR\n" \
  "  This program was created on 16/dec/2006 by J. Stolfi.\n" \
  "WARRANTY\n" \
  argparser_help_info_NO_WARRANTY "\n" \
  "\n" \
  "RIGHTS\n" \
  "  " dm_test_005_coding_C_COPYRIGHT ".\n" \
  "\n" \
  argparser_help_info_STANDARD_RIGHTS  

#include <dm_basic.h>
#include <dm_seq.h>

#include <jsrandom.h>
#include <affirm.h>
#include <argparser.h>

#include <math.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

typedef struct options_t 
  { char *outName;     /* Output file name prefix (minus extensions). */
    int nSamples;      /* Number of samples to test. */
  } options_t;
  
int main(int argc, char**argv);

options_t *dm_get_options(int argc, char**argv);
  /* Parses the command line options, packs 
    them into a {options_t} record. */

int main(int argc, char**argv)
  { 
    options_t *o = dm_get_options(argc, argv);
    int N = o->nSamples;
    int i;
    
    fprintf(stderr, "code value        recodings\n");
    fprintf(stderr, "---- ------------ --------------\n");
    
    /* Encoding consistency check: */
    for (i = 1; i < 256; i++)
      { dm_sample_t ev = i - 128;
        double fv = dm_sample_decode(ev);
        dm_sample_t tva = dm_sample_encode(fv - 1.0e-6*(1 + fabs(fv)));
        dm_sample_t tvb = dm_sample_encode(fv);
        dm_sample_t tvc = dm_sample_encode(fv + 1.0e-6*(1 + fabs(fv)));
        fprintf(stderr, "%+04d %12.8f %+04d %+04d %+04d\n", ev, fv, tva, tvb, tvc);
        assert((ev == tva) && (ev == tvb) && (ev == tvc));
      }
    
    /* Encoding error test: */
    double sumd = 0;
    double sumd2 = 0;
    int hist[256];
    for (i = 0; i < 256; i++) { hist[i] = 0; }
    int k;
    for (k = 0; k < N; k++)
      { /* Generate a random normal-distributed value {dv}: */
        double rv = dgaussrand();
        /* Encode it: */
        dm_sample_t ev = dm_sample_encode(rv);
        /* Histogram count: */
        hist[ev + 128] += 1;
        /* Decode the encoded value: */
        double dv = dm_sample_decode(ev);
        /* Accumulate the value and squared value of error: */
        double dif = dv - rv;
        sumd += dif;
        sumd2 = dif*dif;
      }
    /* Compute and print the average and root-mean-square error: */
    double avg_err = sumd/N;
    double rms_err = sqrt(sumd2/N);
    fprintf(stderr, "avg error = %+12.4e\n", avg_err);
    fprintf(stderr, "rms error = %+12.4e\n", rms_err);
    
    /* Print encoded value histogram: */
    fprintf(stderr, "code count prob\n");
    fprintf(stderr, "---- ----- -------\n");
    for (i = 0; i < 256; i++)
      { dm_sample_t ev = i - 128;
        fprintf(stderr, "%+04d %5d %7.5f\n", ev, hist[i], ((double)(hist[i]))/N);
      }

    /* ...and we are done: */
    fprintf(stderr, "done.\n");
    return 0;
  }
  
options_t* dm_get_options(int argc, char**argv)
  { options_t *o = (options_t *)notnull(malloc(sizeof(options_t)), "no mem");
    
    argparser_t *pp = argparser_new(stderr, argc, argv);
    argparser_set_help(pp, PROG_HELP);
    argparser_set_info(pp, PROG_INFO);
    argparser_process_help_info_options(pp);
    
    argparser_skip_parsed(pp);
    
    o->nSamples = argparser_get_next_int(pp, 1, INT_MAX);
    o->outName = argparser_get_next(pp);

    argparser_finish(pp);
    
    return o;
  }
