#define PROG_NAME "dm_test_180_discrim"
#define PROG_DESC "analysis of true/false candidate discrimination"
#define PROG_VERS "1.0"

/* Last edited on 2008-04-19 23:30:21 by stolfi */

#define dm_test_180_discrim_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#define PROG_HELP \
  PROG_NAME " \\\n" \
  "  -nCands {N_CANDS} \\\n" \
  "  -nBases {N_BASES} \\\n" \
  "  -nLengths {N_LENGTHS} \\\n" \
  "  -mutProb {MUT_PROB} -delProb {DEL_PROB} \\\n" \
  "  -maxLevel {MAX_LEVEL} -minSamples {MIN_SAMPLES} \\\n" \
  "  -initFilter " dm_weight_args_HELP " \\\n" \
  "  -incrFilter " dm_weight_args_HELP " \\\n" \
  "  -nsub {N_SUB} \\\n" \
  "  -scores " dm_score_args_HELP " \\\n" \
  "  [ -delta {DELTA} ] [ -kappa {KAPPA} ] [ -maxUnp {MAX_UNP} ] \\\n" \
  "  {OUT_NAME}"

#define PROG_INFO \
  "NAME\n" \
  "  " PROG_NAME " - " PROG_DESC "\n" \
  "\n" \
  "SYNOPSIS\n" \
  "  " PROG_HELP "\n" \
  "\n" \
  "DESCRIPTION\n" \
  "  This program obtains experimental data showing how the scores" \
  " of homologous and random pairings vary as a function of the matched" \
  " sequence length, at various filtering scales.\n" \
  "\n" \
  "  The program first generates two homologous `genomes', each being a DNA" \
  " nucleotide string derived by simulated mutation from a single random genome; and" \
  " filters those two genomes at various scales of resolution.\n" \
  "\n" \
  "  For each scale, the program then generates a number of disjoint" \
  " pairings between substrings of of those two genomes.  Half of these" \
  " pairings are `true' and half are `false'.  A true pairing" \
  " connects two substrings that descend from the same substring of the `ancestral'" \
  " genome.  A false pairing connects two completely unrelated substrings," \
  " one from each genome.\n" \
  "\n" \
  "  The program then trims each of these `master' pairings to various" \
  " lengths, obtaining a set of nested sub-pairings.  Each of these" \
  " sub-pairings is refined (once), and its length and score" \
  " are written out to disk.\n" \
  "\n" \
  "OUTPUT FILES\n" \
  "  All output files will have names starting with {OUT_NAME}.\n" \
  "\n" \
  "  The main output files are called \"{OUT_NAME}-{LL}-{K}.dat\" where" \
  " {LL} is a two digit filtering level, and {K} indicates the kind" \
  " of pairing (\"t\" for true pairings, \"f\" for false).  In these files," \
  " each line contains the following fields\n" \
  "\n" \
  "    {NX} number of positions in the X sequence that are spanned by the sub-pairing.\n" \
  "    {NY} number of positions in the Y sequence that are spanned by the sub-pairing.\n" \
  "    {SCORE} the score of the pairing.\n" \
  "\n" \
  "  The data for all sub-pairings of the same master pairing appear" \
  " in consecutive lines.  For the sake of {gnuplot}, blank lines separate" \
  " data from different master pairings.\n" \
  "\n" \
  "  The program also calls {} to write out the master pairings for each" \
  " level {LL} and eack kind {K}, to a file called \"{OUT_NAME}-{LL}-{K}.cdv\"," \
  " using the procedure {msm_cand_vec_write}.\n" \
  "\n" \
  "  The program also writes an image file \"{OUT_NAME}-{LL}-eq.pgm\" that" \
  " shows the base-to-base distance matrix for the two genomes." \
  "\n" \
  "OPTIONS\n" \
  "  -nCands {N_CANDS}\n" \
  "    This mandatory argument specifies how many master pairings" \
  " should be generated for each level and kind.\n" \
  "\n" \
  "  -nBases {N_BASES}\n" \
  "    This mandatory argument specifies the approximate number" \
  " of nucleotide bases spanned by the master pairings" \
  " on each genome, at scale 0 (unfiltered).\n" \
  "\n" \
  "  -nLengths {N_LENGTHS}\n" \
  "    This mandatory argument specifies the number of nested sub-pairings" \
  " to be carved out from each master pairing.\n" \
  "\n" \
  "  -mutProb {MUT_PROB}\n" \
  "    This mandatory argument specifies the" \
  " nucleotide mutation probability used when" \
  " generating the two sequences.\n" \
  "\n" \
  "  -delProb {DEL_PROB}\n" \
  "    This mandatory argument specifies the" \
  " nucleotide deletion or insertion probability used" \
  " when generating the two sequences.\n" \
  "\n" \
  "  -maxLevel {MAX_LEVEL}\n" \
  "    This mandatory argument specifies the" \
  " maximum level of filtering to be" \
  " applied.  Level 0 is the unfiltered" \
  " sequence, and each subsequent level is sampled at half" \
  " the frequency of the previous one.\n" \
  "\n" \
  "  -minSamples {MIN_SAMPLES}\n" \
  "    This mandatory argument specifies the minimum number of actual" \
  " data samples (not just sequence positions) in the test sequences, at" \
  " any scale, which are needed for meaningful analysis.  The log" \
  " of the ratio between {N_BASES} and {MIN_SAMPLES} determines" \
  " the maximum filtering level at which the test is performed.\n" \
  "\n" \
  "  -initFilter " dm_weight_args_HELP " \n" \
  "  -incrFilter " dm_weight_args_HELP " \n" \
  "    These mandatory arguments specify the" \
  " weights of the filter to be used at the" \
  " first filtering step and at subsequent" \
  " filtering steps.  " dm_weight_args_norm_sum_INFO "\n" \
  "\n" \
  "  -nsub {N_SUB}\n" \
  "    This mandatory argument specifies the number" \
  " of matching points per sample in the filtered sequences.\n" \
  "\n" \
  "  -scores " dm_score_args_HELP "\n" \
  "    This mandatory argument specifies the" \
  " scoring of steps in a pairing.  " dm_score_args_INFO "\n" \
  "\n" \
  "  -delta {DELTA}\n" \
  "    This optional argument specifies the" \
  " amount of adjustment allowed for the X and Y coordinates of" \
  " internal rungs of each pairing.  The default is 3.\n" \
  "\n" \
  "  -kappa {DELTA}\n" \
  "    This optional argument specifies the" \
  " amount of X and Y extension allowed" \
  " at either end of each pairing.  The default is 6.\n" \
  "\n" \
  "  -maxUnp {MAX_UNP}\n" \
  "    This optional argument specifies !!!.  The default is !!!.\n" \
  "\n" \
  argparser_help_info_HELP_INFO "\n" \
  "SEE ALSO\n" \
  "  dm_match(1)\n" \
  "\n" \
  "AUTHOR\n" \
  "  This program was created on 21/dec/2006 by J. Stolfi.\n" \
  "WARRANTY\n" \
  argparser_help_info_NO_WARRANTY "\n" \
  "\n" \
  "RIGHTS\n" \
  "  " dm_test_180_discrim_C_COPYRIGHT ".\n" \
  "\n" \
  argparser_help_info_STANDARD_RIGHTS  

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_nucleic.h>
#include <dm_score.h>
#include <dm_weight.h>
#include <dm_filter.h>
#include <dm_test_tools.h>

#include <msm_test_tools.h>
#include <msm_image_tools.h>
#include <msm_image.h>

#include <msm_rung.h>
#include <msm_pairing.h>
#include <msm_cand.h>
#include <msm_cand_refine.h>
#include <msm_dyn.h>
#include <msm_multi.h>

#include <affirm.h>
#include <float_image.h>
#include <float_pnm_image.h>
#include <jspnm_image.h>
#include <jsrandom.h>
#include <jsfile.h>
#include <argparser.h>

#include <math.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

typedef struct options_t 
  { /* Sequence generation parameters: */
    double nBases;       /* Expected number of bases. */
    double mutProb;      /* Mutation probability. */
    double delProb;      /* Insertion/deletion probability. */
    /* Candidate generation parameters: */
    int nCands;          /* Number of master candidates to generate. */
    int nLengths;        /* Number of sub-candidates to cut out from each master cand. */
    /* Filtering parameters: */
    int maxLevel;        /* Maximum filtering level. */
    int minSamples;      /* Min samples in a full-length candidate. */
    double_vec_t w0;     /* Filter weights for first stage (level 0 to level 1). */
    double_vec_t w1;     /* Filter weights for subsequent stages. */
    int nsub;            /* Number of matching points per datum in the filtered sequences. */
    /* Scoring parameters: */
    dm_score_args_t sc;  /* Scoring criterion. */
    /* Refinement parameters: */
    int delta;           /* Half-width of tableau around original pairing. */
    int kappa;           /* Extension of tableau beyond ends of pairing. */
    int maxUnp;          /* !!! WHAT IS THIS ??? */
    /* Output parameters: */
    char *outName;       /* Output file name prefix (minus extensions). */
  } options_t;
  
int main(int argc, char**argv);

options_t *dm_td_get_options(int argc, char**argv);
  /* Parses the command line options, packs 
    them into a {options_t} record. */

void dm_td_generate_genome_pair_multi
  ( int totBases,
    options_t *o, 
    dm_seq_t xf[], 
    dm_seq_t yf[], 
    msm_pairing_t *pf[],
    int minSamples,
    int *maxUsefulLevel
  );
  /* Generates an homologous pair of DNA sequences {x,y} with
    {~totBases} bases in each sequence; then, for each scale {k} in
    {0..o->maxScale}, computes datum sequences {xf[k],yf[k]}
    corresponding to {x,y} filtered to scale {k}. Also returns the
    corresponding ``true'' pairings {*(pf[k])} between those
    sequences (as recorded during the pair generation pocess). Then
    sets {*maxUsefulLevel} to the maximum scale {k} where the
    sequences have at least {minSamples} datums.
    
    The parameters for sequence pair generation, filtering, and max
    scale slection are taken from the options record {*o}. */

void dm_td_for_level
  ( options_t *o,
    char *prefix, 
    int level,
    dm_seq_t *x, 
    dm_seq_t *y,
    msm_pairing_t *p
  );
  /* Generates discrimination data for the homologous genomes
    {x,y}, related by the pairing {p}, filtered at some scale. 
    
    The procedure writes an image file called
    "{prefix}-{LL}-eq.ppm" where {LL} is the two-digit {level},
    showing the nucleotide distance matrix between the genomes {x} and
    {y}, and the homology pairing {p}. Then calls
    {dm_td_for_level_kind(o,"{prefix}-{LL}",homo,x,y,p)},
    first with {homo=FALSE}, then with {homo=TRUE}. */

void dm_td_for_level_kind
  ( options_t *o, 
    char *prefix, 
    bool_t homo,
    dm_seq_t *x, 
    dm_seq_t *y, 
    msm_pairing_t *p
  );
  /* Generates discrimination data for filtering level {level}
    and candidates of kind {homo} (FALSE=random, TRUE=homologous). 
    
    The procedure generates {o->nCands} disjoint
    master candidates between the strings {*x,*y}, according to the
    parameters in {o}. Each homologous candidate spans two string
    segments that are exactly matched by the pairing {*p}.
    Each random candidate spans segments that are completely
    un-matched by that pairing.  Each master candidate 
    spans about {1/(o->nCands+1)} of {p}'s span.
    
    Writes the master candidates to file "{prefix}-{K}.cdv" 
    as per {msm_cand_vec_write}; where {K} is "f" for false
    candidates, "t" for true ones.  Then extracts {o->nLengths} sub-candidates
    from each master candidate, refines them, and writes their lengths
    and scores to a data file named "{prefix}-{K}.dat".
    
    ??? CHECK REQUIREMENTS FOR MONOTONICITY !!! */

void dm_td_split_pairing(msm_pairing_t *p, int nSegs, msm_pairing_t *pc[]);
  /* Splits the pairing {p} into {nSegs} disjoint sub-pairings, which are
    returned in {pc[0..nSegs-1]}.  The pairing must be increasing.  The
    pieces will be open even if {p} is circular. */

msm_cand_vec_t dm_td_generate_cands
  ( int nCands, 
    bool_t homo, 
    dm_seq_t *x, 
    dm_seq_t *y, 
    msm_pairing_t *p
  );
  /* Generates a list with {nCands} candidates of type {homo}
    (FALSE=random, TRUE=homologous) between genomes {x,y}, whose
    homology is specified by the pairing {p}. Each candidate will be
    increasing, atomic, and will span approximately {np/(nCands+1)}
    matching positions in each sequence, where {np} is the number of
    positions in that sequence. */

msm_pairing_t *dm_td_make_cross_pairing
  ( dm_seq_t *x, 
    msm_pairing_t *xp, 
    dm_seq_t *y, 
    msm_pairing_t *yp
  );
  /* Creates a pairing between sequences {x} and {y}, which pairs the
    positions of {x} spanned by {xp} on its X side with the positions
    of {y} spanned by {yp} on its Y side. The pairing will be
    increasing, but will have only two rungs. */

void dm_td_analyze_candidate
  ( FILE *wr, 
    options_t *o,
    dm_seq_t *x, 
    dm_seq_t *y, 
    msm_cand_t *cd,
    msm_dyn_tableau_t *tb
  );
  /* Computes the score of {o->nLengths} initial sub-candidates of candidate {cd},
    assumed to be a pairing between sequences {x} and {y}.  
    
    The pairing of {cd} is must be increasing and atomic. The total
    span of sub-candidate {i} will be approximately
    {i*tp/(o->nLengths)} where {tp} is the total span of {cd}.
    
    For each sub-candidate, the procedure computes the optimum pairing
    {p} between the two sides and writes one data line to {wr}. The
    line will contain the counts {NX} and {NY} of positions spanned on
    each sequence by the optimum pairing {p}, and its score {SCORE}.
    
    The pairing {p} and its score are found with {msm_cand_refine}
    with {shrink=FALSE} and other parameters taken from {o}.  The tableau
    {tb} is a work area that must be created by the client and is enlarged
    by the procedure as needed. */    

int main(int argc, char**argv)
  { 
    options_t *o = dm_td_get_options(argc, argv);
    
    /* Compute length of master sequence pair: */
    int totBases = o->nBases*(o->nCands + 2); /* Should be enough. */
    
    /* Generate master genome pair at all scales: */
    fprintf(stderr, "generating and writing the homologous seqs at various scales ...\n");
    dm_seq_t xf[o->maxLevel + 1];  /* Left sequence of genome at each scale. */
    dm_seq_t yf[o->maxLevel + 1];  /* Right sequence of genome at each scale. */
    msm_pairing_t *pf[o->maxLevel + 1];  /* Homology pairing between the two genomes. */
    int maxUsefulLevel; /* Max filtering level worth testing. */
    int minSamples = o->minSamples*(o->nCands + 2); /* Min samples in genome. */
    dm_td_generate_genome_pair_multi(totBases, o, xf, yf, pf, minSamples, &maxUsefulLevel);
    
    fprintf(stderr, "running analyses ...\n");
    int level;
    for (level = 0; level <= maxUsefulLevel; level++)
      { /* Perform tests for both types of candidate: */
        dm_td_for_level(o, o->outName, level, &(xf[level]), &(yf[level]), pf[level]);
      }
  
    /* ...and we are done: */
    fprintf(stderr, "done.\n");
    return 0;
  }

void dm_td_for_level
  ( options_t *o,
    char *prefix, 
    int level, 
    dm_seq_t *x, 
    dm_seq_t *y,
    msm_pairing_t *p
  )
  {
    /* Prefix for file names: */
    char *levPrefix = NULL;
    asprintf(&levPrefix, "%s-%02d", prefix, level);
    
    auto double rung_score(msm_seq_desc_t *xp, msm_seq_desc_t *yp, msm_rung_t g);

    double rung_score(msm_seq_desc_t *xp, msm_seq_desc_t *yp, msm_rung_t g)
      { msm_seq_desc_same_seq(xp, &(x->sd), TRUE);
        msm_seq_desc_same_seq(yp, &(y->sd), TRUE);
        return dm_score_rung(&(o->sc), g, x, y);
      }

    fprintf(stderr, "plotting distance matrix for test seq pair ...\n");
    msm_image_seq_seq_score_write_named(&(x->sd), &(y->sd), rung_score, p, levPrefix, "-eq");

    /* Perform tests for both types of candidate: */
    dm_td_for_level_kind(o, levPrefix, FALSE, x, y, p);
    dm_td_for_level_kind(o, levPrefix, TRUE,  x, y, p);
    
    free(levPrefix);
  }
  
void dm_td_for_level_kind
  ( options_t *o, 
    char *prefix, 
    bool_t homo, 
    dm_seq_t *x, 
    dm_seq_t *y, 
    msm_pairing_t *p
  )
  {
    int nCands = o->nCands;
    
    /* Prefix for file names: */
    char *homoPrefix = NULL;
    asprintf(&homoPrefix, "%s-%c", prefix, "ft"[homo]);
    
    /* Extracts the required number of paired segments from genome pair {x,y}: */
    msm_cand_vec_t cdv = dm_td_generate_cands(nCands, homo, x, y, p);
    msm_cand_vec_write_named(&cdv, homoPrefix, "");
    
    /* Open the output data file: */
    FILE *fdat = msm_open_write(homoPrefix, "", ".dat", TRUE);
    
    /* Allocate a work area for {msm_cand_refine}: */
    msm_dyn_tableau_t tb = msm_dyn_tableau_new();
    
    /* Test the candidates truncated at several lengths: */
    int icd;
    for (icd = 0; icd < nCands; icd++)
      { dm_td_analyze_candidate(fdat, o, x, y, &(cdv.e[icd]), &tb);
        /* Write a blank line between cands, for {gnuplot}'s sake: */
        fprintf(fdat, "\n");
      }
      
    fclose(fdat);
    free(homoPrefix);
    msm_cand_vec_free(&cdv);
    msm_dyn_tableau_free(&tb);
  }
  
msm_cand_vec_t dm_td_generate_cands
  ( int nCands, 
    bool_t homo, 
    dm_seq_t *x, 
    dm_seq_t *y, 
    msm_pairing_t *p
  )  
  {
    demand(! msm_pairing_is_circular(p), "pairing must be open");
    demand(! x->sd.circ, "X seq must be open");
    demand(! y->sd.circ, "Y seq must be open");
    
    /* Compute the number {nSegs} of pieces into which {x,y,p} will be split: */
    int nSegs = nCands + 1;
    
    /* Divide {p} into {nSegs} pieces {*pc[i]}, each with span {~cdSpan}: */
    msm_pairing_t *pc[nSegs];
    dm_td_split_pairing(p, nSegs, pc);
    
    /* Generate the candidates: */
    msm_cand_vec_t cdv = msm_cand_vec_new(nCands);
    int icd;
    for (icd = 0; icd < nCands; icd++)
      { msm_pairing_t *pr; /* Raw pairing for candidate. */
        if (homo)
          { /* Create a candidate from segment {icd} of the pairing: */
            pr = msm_pairing_copy(pc[icd]);
          }
        else
          { /* Create a candidate that pairs two totally unrelated segments of {x,y}: */
            /* Choose a segment {xseg} in the X sequence: */
            int xseg = icd;
            assert((xseg >= 0) && (xseg < nSegs));
            /* Choose the Y segment {yseg != xseg} that is to be paired with it: */
            int yseg;
            yseg = abrandom(0,nSegs-2);
            if (yseg >= xseg) { yseg++; }
            assert((yseg >= 0) && (yseg < nSegs));
            assert(xseg != yseg);
            /* Create a candidate for these two segments, with trivial pairing: */
            pr = dm_td_make_cross_pairing(x, pc[xseg], y, pc[yseg]);
          }
        /* Interpolate {pr} to obtain an atomic pairing {pi}: */
        msm_pairing_t *pi = msm_pairing_interpolate(pr);
        /* Turn pairing {pi} into a candidate: */
        cdv.e[icd] = msm_cand_from_pairing(&(x->sd), &(y->sd), pi, 0.0);
        msm_pairing_free(pr); 
      }
    return cdv;
  }

void dm_td_split_pairing(msm_pairing_t *p, int nSegs, msm_pairing_t *pc[])
  {
    (void)msm_pairing_is_increasing(p, TRUE);
    
    /* Get the count of fundamental rungs: */
    int nRungs = msm_pairing_fund_rung_count(p);

    /* Get the total span of {p} on the X and Y sequences, not counting the loopback step: */
    int totSpan = msm_pairing_sub_span(p, 0, nRungs-1, -1);
    
    /* Choose the total span {cdSpan} of each piece (and hence of each candidate): */
    int cdSpan = totSpan/nSegs;
    
    fprintf(stderr, "splitting a pairing with %d rungs into %d segments\n", nRungs, nSegs);
    int k = 0; /* Index of current rung in pairing {p}. */
    int iseg;
    for (iseg = 0; iseg < nSegs; iseg++)
      { assert(k < nRungs);
        int kini = k;
        while ((k < nRungs) && (msm_pairing_sub_span(p, kini, k, -1) <= cdSpan)) { k++; }
        int kfin = k - 1;
        assert(kini <= kfin);

        /* Show the selected segment"*/
        fprintf(stderr, "  segment %d = p[%d..%d]", iseg, kini, kfin);
        msm_rung_t gini = msm_pairing_get_rung(p, kini);
        msm_rung_t gfin = msm_pairing_get_rung(p, kfin);
        fprintf(stderr, "  spans X[%d..%d] (%d)", gini.c[0], gfin.c[0], gfin.c[0]-gini.c[0]+1);
        fprintf(stderr, " + Y[%d..%d] (%d)", gini.c[1], gfin.c[1], gfin.c[1]-gini.c[1]+1);

        pc[iseg] = msm_pairing_sub_copy(p, kini, kfin);
        assert(msm_rung_equal(gini, msm_pairing_get_rung(pc[iseg], 0)));
        assert(msm_rung_equal(gfin, msm_pairing_get_rung(pc[iseg], kfin-kini)));
        fprintf(stderr, " = (%d)\n", msm_pairing_span(pc[iseg], -1));
        assert(! msm_pairing_is_circular(pc[iseg]));
      }
  }

msm_pairing_t *dm_td_make_cross_pairing
  ( dm_seq_t *x, 
    msm_pairing_t *xp, 
    dm_seq_t *y, 
    msm_pairing_t *yp
  )
  {
    /* Obtain the initial and final rungs {gini,gfin} of the desired pairing: */
    msm_rung_t gini, gfin;
    int j;
    for (j = 0; j < 2; j++)
      { /* Grab the whole sequence {s} on side {j} and its position count {ns}: */
        dm_seq_t *s = (j == 0 ? x : y);
        demand(! s->sd.circ, "seq must be open");
        int ns = s->sd.npos;
        /* Grab the pairing {p} that defines the segm on {s}, and its rung count {ng}: */
        msm_pairing_t *p = (j == 0 ? xp : yp);
        demand(! msm_pairing_is_circular(p), "pairing must be open");
        int ng = msm_pairing_fund_rung_count(p);
        /* Set the indices of first and last spanned positions on {s}: */
        gini.c[j] = msm_pairing_get_rung(p, 0).c[j];
        demand((gini.c[j] >= 0) && (gini.c[j] < ns), "initial rung out of bouds");
        gfin.c[j] = msm_pairing_get_rung(p, ng-1).c[j];
        demand((gfin.c[j] >= 0) && (gfin.c[j] < ns), "final rung out of bounds");
      }
    /* Create a trivial pairing from {gini,gfin}: */
    msm_rung_vec_t gvt = msm_rung_vec_new(2);
    gvt.e[0] = gini; 
    gvt.e[1] = gfin;
    return msm_pairing_from_rung_vec(&gvt, FALSE);
  }

void dm_td_analyze_candidate
  ( FILE *wr, 
    options_t *o,
    dm_seq_t *x, 
    dm_seq_t *y, 
    msm_cand_t *cd,
    msm_dyn_tableau_t *tb
  )
  {
    int nLengths = o->nLengths;
    
    /* Make sure that {cd}'spairing is atomic and increasing: */
    (void)msm_pairing_is_atomic_increasing(cd->pr, TRUE);
    
     auto double step_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g0, msm_rung_t g1);
      /* Step scoring function. */

    double step_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g0, msm_rung_t g1)
      { (void)msm_seq_desc_same_seq(&(x->sd), ad, TRUE); 
        (void)msm_seq_desc_same_seq(&(y->sd), bd, TRUE); 
        return dm_score_step(&(o->sc), g0, g1, x, y);
      }
    
    /* Compute the total span of the candidate on the two sequences: */
    int totSpan = msm_pairing_span(cd->pr, -1);
    
    fprintf(stderr, "analyzing candidate with span = %d\n", totSpan);
    
    /* Compute the index {mid} of the middle rung of {cd}: */
    int mid = msm_pairing_fund_rung_count(cd->pr)/2;
    int ilen;
    for (ilen = 0; ilen < nLengths; ilen++)
      { /* Compute desired total span of this sub-candidate: */
        int segSpan = (((int64_t)(ilen + 1))*((int64_t)totSpan))/nLengths ;
        /* Carve a centered sub-pairing {ps} of {cd->pr} with total span {segSpan}: */
        fprintf(stderr, "  carving piece with span %d", segSpan);
        msm_pairing_t *ps = msm_pairing_trim_to_span(cd->pr, -1, segSpan, mid, 00);
        int ngs = msm_pairing_fund_rung_count(ps);
        msm_rung_t gini = msm_pairing_get_rung(ps, 0);
        msm_rung_t gfin = msm_pairing_get_rung(ps, ngs - 1);
        fprintf(stderr, "  spans %d..%d on X seq", gini.c[0], gfin.c[0]);
        fprintf(stderr, "  spans %d..%d on Y seq", gini.c[1], gfin.c[1]);
        fprintf(stderr, "  total span = %d positions\n", msm_pairing_span(ps, -1));
        /* Make it into a candidate {cds}: */
        msm_cand_t cds = msm_cand_from_pairing(&(x->sd), &(y->sd), ps, 0.0);
        /* Refine the candidate: */
        msm_cand_t cdf = msm_cand_refine
          ( &cds, o->delta, o->kappa, FALSE, FALSE, o->maxUnp, step_score, tb );
        /* Write X span, Y span, and score to plot-data file: */
        int xSpan = msm_pairing_span(cdf.pr, 0);
        int ySpan = msm_pairing_span(cdf.pr, 1);
        fprintf(wr, "%05d %05d %12.4f\n", xSpan, ySpan, cdf.score);
        msm_cand_free(&cds);
        msm_cand_free(&cdf);
      }
    fflush(wr);        
  }

void dm_td_generate_genome_pair_multi
  ( int totBases,
    options_t *o, 
    dm_seq_t xf[], 
    dm_seq_t yf[], 
    msm_pairing_t *pf[],
    int minSamples,
    int *maxUsefulLevel
  )
  {
    fprintf(stderr, "generating the ancestral sequence ...\n");
    char *borg = dm_nucleic_string_throw(totBases, TRUE, TRUE);
    
    fprintf(stderr, "writing out the ancestral sequence ...\n");
    dm_nucleic_string_write_named(borg, "Ancestral", o->outName, "-org");
    
    fprintf(stderr, "generating and writing the mutated copies ...\n");
    bool_t circ_seq = FALSE; /* TRUE to use circular sequences. */
    dm_seq_t x, y;  /* The two homologous sequences. */
    msm_pairing_t *p; /* The true pairs. */
    dm_test_tools_make_seq_pair
      ( borg, o->mutProb, o->delProb, circ_seq, 0, "X", &x, 1, "Y", &y, &p, o->outName );
    
    fprintf(stderr, "getting the weight tables ...\n");
    char *wname0 = dm_weight_make_descr(&(o->w0), "%6.4f");
    char *wname1 = dm_weight_make_descr(&(o->w1), "%6.4f");

    fprintf(stderr, "trying to filter up to level %d ...\n", o->maxLevel);
    dm_filter_seq_multi(&x, o->maxLevel, &(o->w0), wname0, &(o->w1), wname1, xf, o->nsub);
    dm_filter_seq_multi(&y, o->maxLevel, &(o->w0), wname0, &(o->w1), wname1, yf, o->nsub);
    
    /* Map the pairing {p} to the sucessive scales: */
    pf[0] = msm_pairing_copy(p);
    int nxold = dm_seq_num_positions(&(xf[0]));
    int nyold = dm_seq_num_positions(&(yf[0]));
    int level;
    for (level = 1; level <= o->maxLevel; level++)
      { int nw = (level == 1 ? o->w0.ne : o->w1.ne);
        int nxnew = dm_seq_num_positions(&(xf[level]));
        int nynew = dm_seq_num_positions(&(yf[level]));
        msm_pairing_t *pc = msm_pairing_map_to_coarser
          ( pf[level-1], nxold, nyold, nxnew, nynew, circ_seq, circ_seq, nw );
        pf[level] = msm_pairing_make_increasing(pc, o->nsub);
        msm_pairing_free(pc);
        nxold = nxnew; nyold = nynew;
      }

    /* Figure out max useful level {maxLev}: */
    int maxLev = o->maxLevel;
    while 
      ( (maxLev >= 0) && 
        ( (dm_seq_num_datums(&(xf[maxLev])) < minSamples) ||
          (dm_seq_num_datums(&(yf[maxLev])) < minSamples)
        )
      ) { maxLev--; }
    fprintf(stderr, "maximum useful level is %d\n", maxLev);

    fprintf(stderr, "writing and plotting the filtered sequences ...\n");
    dm_test_tools_seq_multi_write_and_plot_named(xf, maxLev, "X", o->outName, "-X");
    dm_test_tools_seq_multi_write_and_plot_named(yf, maxLev, "Y", o->outName, "-Y");
    (*maxUsefulLevel) = maxLev;
  }
      
options_t *dm_td_get_options(int argc, char**argv)
  { 
    options_t *o = (options_t *)notnull(malloc(sizeof(options_t)), "no mem");
    
    argparser_t *pp = argparser_new(stderr, argc, argv);
    argparser_set_help(pp, PROG_HELP);
    argparser_set_info(pp, PROG_INFO);
    argparser_process_help_info_options(pp);
     
    argparser_get_keyword(pp, "-nCands");
    o->nCands = argparser_get_next_int(pp, 0, INT_MAX);
     
    argparser_get_keyword(pp, "-nLengths");
    o->nLengths = argparser_get_next_int(pp, 0, INT_MAX);
  
    argparser_get_keyword(pp, "-nBases");
    o->nBases = argparser_get_next_int(pp, 1, INT_MAX);
    
    argparser_get_keyword(pp, "-mutProb");
    o->mutProb = argparser_get_next_double(pp, 0.0, 1.0);
    
    argparser_get_keyword(pp, "-delProb");
    o->delProb = argparser_get_next_double(pp, 0.0, 1.0 - o->mutProb);
    
    argparser_get_keyword(pp, "-initFilter");
    o->w0 = dm_weight_args_parse(pp, TRUE);

    argparser_get_keyword(pp, "-incrFilter");
    o->w1 = dm_weight_args_parse(pp, TRUE);

    argparser_get_keyword(pp, "-maxLevel");
    o->maxLevel = argparser_get_next_int(pp, 0, 20);

    argparser_get_keyword(pp, "-minSamples");
    o->minSamples = argparser_get_next_int(pp, 20, INT_MAX);

    argparser_get_keyword(pp, "-nsub");
    o->nsub = argparser_get_next_int(pp, 0, INT_MAX);
    
    argparser_get_keyword(pp, "-scores");
    o->sc = dm_score_args_parse(pp);
    
    if (argparser_keyword_present(pp, "-delta"))
      { o->delta = argparser_get_next_int(pp, 0, INT_MAX); }
    else
      { o->delta = 3; }
    
    if (argparser_keyword_present(pp, "-kappa"))
      { o->kappa = argparser_get_next_int(pp, 0, INT_MAX); }
    else
      { o->kappa = 6; }
    
    if (argparser_keyword_present(pp, "-maxUnp"))
      { o->maxUnp = argparser_get_next_int(pp, 0, INT_MAX); }
    else
      { o->maxUnp = 0; /* !!! FIND PROPER DEFAULT */ }
    
    argparser_skip_parsed(pp);
    
    o->outName = argparser_get_next(pp);

    argparser_finish(pp);
    
    return o;
  }
