/* See {dm_test_tools.h} */
/* Last edited on 2008-04-19 23:15:39 by stolfi */

#define dm_test_tools_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#define _GNU_SOURCE
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <affirm.h>
#include <bool.h>

#include <msm_basic.h>
#include <msm_seq_desc.h>
#include <msm_rung.h>
#include <msm_double_vec.h>
#include <msm_cand_refine.h>
#include <msm_ps_tools.h>
#include <msm_test_tools.h>

#include <dm_test_tools.h>
#include <dm_nucleic.h>
#include <dm_score.h>
#include <dm_seq.h>

msm_cand_t dm_test_tools_get_optimum_pairing
  ( dm_score_args_t *sc, 
    dm_seq_t *xp, 
    dm_seq_t *yp, 
    int delta, 
    msm_dyn_tableau_t *tb,
    int maxIter
  ) 
  { 
    auto double step_score
      ( msm_seq_desc_t *xd, 
        msm_seq_desc_t *yd,
        msm_rung_t g0,
        msm_rung_t g1
      );
    /* Assumes that {xd} and {yd} are the descriptors of {xp} and {yp},
      and tthat {g0} and {g1} are rungs between the 
      two sequences.  Returns the score of the step {g0->g1}. */
    
    return  msm_test_tools_get_optimum_pairing
      ( &(xp->sd), &(yp->sd), delta, &step_score, tb, maxIter );

    double step_score
      ( msm_seq_desc_t *xd, 
        msm_seq_desc_t *yd,
        msm_rung_t g0,
        msm_rung_t g1
      )
      { (void)msm_seq_desc_same_seq(xd, &(xp->sd), TRUE); 
        (void)msm_seq_desc_same_seq(yd, &(yp->sd), TRUE); 
        return dm_score_step(sc, g0, g1, xp, yp);
      }
  }

void dm_test_tools_seq_write_and_plot_named
  ( dm_seq_t *seq,
    char *title,
    char *name,
    char *tag
  )
  {
    fprintf(stderr, "sequence = "); 
    msm_seq_desc_write(stderr, " ( ", &(seq->sd), 1, 1, 4, " )"); 
    fprintf(stderr, "  samples = %4d  nsub = %2d\n", seq->dv.ne, seq->nsub);

    /* Check whether the sample vector is compatible with the seq descriptor: */
    bool_t circ = seq->sd.circ;
    int nsmp = seq->dv.ne;
    int npos = (circ ? seq->nsub*nsmp : seq->nsub*(nsmp-1)+1);
    demand(dm_seq_num_positions(seq) == npos, "inconsistent sequence descriptor");
    
    /* Ignore the {seq} for now: */
    dm_seq_write_named(seq, name, tag);
    
    /* Plotting: */
    msm_ps_tools_t *mps = msm_ps_tools_new_graph
      ( NULL, name, tag, 
        /*hGraphSize:*/ 165.0, 
        /*vGraphSize:*/  25.0, 
        /*scaleL:*/ FALSE, /*titleL:*/ FALSE, 
        /*scaleR:*/ FALSE, /*titleR:*/ FALSE,
        /*scaleB:*/ FALSE, /*titleB:*/ FALSE,
        /*scaleT:*/ FALSE, /*titleT:*/ FALSE, 
        /*fontSize:*/ dm_test_tools_FONT_SIZE,
        /*maxLabChars:*/ 5,
        /*mrg:*/ 1.0
      );
      
    /* Repack the samples as a single vector {smp[]}, for {msm_ps_tools_draw_graphs}: */
    int nplt = (circ ? nsmp + 1 : nsmp); /* Number of samples for plotting. */
    double smp[dm_CHANNELS*nplt]; 
    int i, c;
    for (i = 0; i < nplt; i++)
      for (c = 0; c < dm_CHANNELS; c++)
        { smp[nplt*c + i] = dm_seq_get_sample(seq, i, c); }

    msm_ps_tools_draw_graphs
      ( mps, dm_CHANNELS, nsmp, circ, 
        NULL, smp, +INF, -INF
      );
  }

void dm_test_tools_seq_multi_write_and_plot_named
  ( dm_seq_t seq[],
    int maxLevel,
    char *title,
    char *name,
    char *tag
  )
  { int level;
    for (level = 0; level <= maxLevel; level++)
      { char *titlei = NULL; 
        asprintf(&titlei, "%s level %02d", title, level);
        char *tagi = NULL; 
        asprintf(&tagi, "%s-%02d", tag, level);
        dm_test_tools_seq_write_and_plot_named(&(seq[level]),titlei,name,tagi);
        free(titlei); free(tagi);
      }
  }

#define dm_rung_huge (msm_rung_t){{ INT_MAX, INT_MAX }}

void dm_test_tools_make_seq_pair
  ( char *borg, 
    double mutProb, 
    double delProb, 
    bool_t circ,
    dm_seq_id_t xid, 
    char *xtag,
    dm_seq_t *xP,
    dm_seq_id_t yid, 
    char *ytag,
    dm_seq_t *yP,
    msm_pairing_t **prP,
    char *outName
  )
  { /* Generate two mutated copies of {b}, save the indices of copied chars: */
    char *xdna, *ydna; /* The mutated copies. */
    msm_rung_vec_t xgv, ygv;  /* Index pairs for letters that were copied. */
    dm_nucleic_string_mutate(borg, TRUE, TRUE, mutProb, delProb, &xdna, &xgv);
    dm_nucleic_string_mutate(borg, TRUE, TRUE, mutProb, delProb, &ydna, &ygv);
    /* Merge the rung vectors {xgv,ygv} by component 0 to create the pairing {*prP}: */
    int ngmax = (xgv.ne < ygv.ne ? xgv.ne : ygv.ne);
    msm_rung_vec_t gv = msm_rung_vec_new(ngmax);
    int ng = 0; /* Output rungs will be {gv[0..ng-1]}. */
    int kx = 0, ky = 0;
    while ((kx < xgv.ne) && (ky < ygv.ne))
      { msm_rung_t xg = (kx >= xgv.ne ? dm_rung_huge : xgv.e[kx]);
        msm_rung_t yg = (ky >= ygv.ne ? dm_rung_huge : ygv.e[ky]);
        if (xg.c[0] < yg.c[0]) 
          { kx++; }
        else if (xg.c[0] > yg.c[0])
          { ky++; }
        else
          { msm_rung_t g = (msm_rung_t){{ xg.c[1], yg.c[1] }};
            if (ng > 0) { (void)msm_rung_step_is_increasing(gv.e[ng-1], g, /*die*/ TRUE); }
            msm_rung_vec_expand(&gv, ng);
            gv.e[ng] = g; ng++; 
            kx++; ky++; 
          }
      }
    demand(ng > 0, "empty output pairing");
    if (circ)
      { /* Add the closing rung: */
        msm_rung_t gini = gv.e[0];
        int nx = strlen(xdna);
        int ny = strlen(ydna);
        msm_rung_t grep = (msm_rung_t){{ gini.c[0] + nx, gini.c[1] + ny }};
        (void)msm_rung_step_is_increasing(gv.e[ng-1], grep, /*die*/ TRUE);
        msm_rung_vec_expand(&gv, ng);
        gv.e[ng] = grep; ng++; 
      }
    msm_rung_vec_trim(&gv, ng);
    /* Generate pairing from rungs: */
    msm_pairing_t *pr = msm_pairing_from_rung_vec(&gv, circ);
    /* Generate comment: */
    char *cmt = NULL;
    asprintf(&cmt, "Mutated mp = %8.6f dp = %8.6f\n", mutProb, delProb);
    (*xP) = dm_seq_from_nucleic_string(xid, xtag, cmt, xdna, circ);
    (*yP) = dm_seq_from_nucleic_string(yid, ytag, cmt, ydna, circ);
    (*prP) = pr;
    /* Write sequences to disk: */
    dm_test_tools_write_generated_sequence(xdna, xP, outName);
    dm_test_tools_write_generated_sequence(ydna, yP, outName);
    free(xdna); free(xgv.e);
    free(ydna); free(ygv.e);
  }

void dm_test_tools_write_generated_sequence(char *b, dm_seq_t *s, char *outName)
  { /* Write sequence to disk: */
    char *fileTag = NULL;
    asprintf(&fileTag, "-%s", s->sd.name);
    dm_nucleic_string_write_named(b, s->cmt, outName, fileTag);
    dm_seq_write_named(s, outName, fileTag);
    free(fileTag);
  }
  

