/* See {image_coords.h}. */
/* Last edited on 2007-09-06 18:25:47 by stolfi */

#include <bool.h>
#include <r2.h>
#include <r3x3.h>
#include <hr2.h>
#include <argparser.h>
#include <image_coords.h>

#define BIG  (1.0e+100)
  /* A very large value, but still far from overflow. */

hr2_pmap_t imgc_coord_sys_map
  ( bool_t hrev, 
    bool_t vrev, 
    bool_t center, 
    r2_t *org, 
    int cols, 
    int rows
  )
  {
    /* Start with the identity matrix {A}, get its weight {w}: */
    r3x3_t A; r3x3_ident(&A);
    double w = A.c[0][0];
    
    /* Apply the axis-reversal and default origin changes: */
    if (hrev) { A.c[1][1] = -w; A.c[0][1] = w*cols; }
    if (vrev) { A.c[2][2] = -w; A.c[0][2] = w*rows; }
    
    /* Apply the origin shift: */
    if (center) 
      { A.c[0][1] -= 0.5*w*cols; A.c[0][2] -= 0.5*w*rows; }
    else
      { A.c[0][1] -= w*org->c[0]; A.c[0][2] -= w*org->c[1]; }
      
    /* Build the map: */
    hr2_pmap_t M;
    M.dir = A; r3x3_inv(&A, &(M.inv));
    return M;
  }

void imgc_parse_haxis(argparser_t *pp, bool_t *hleft)
  {
    if (argparser_keyword_present(pp, "-haxis"))
      { if (argparser_keyword_present_next(pp, "left"))
          { (*hleft) = TRUE; }
        else if (argparser_keyword_present_next(pp, "right"))
          { (*hleft) = FALSE; }
        else
          { argparser_error(pp, "invalid horizontal axis direction"); }
      }
  }
  
void imgc_parse_vaxis(argparser_t *pp, bool_t *vdown)
  {
    if (argparser_keyword_present(pp, "-vaxis"))
      { if (argparser_keyword_present_next(pp, "up"))
          { (*vdown) = FALSE; }
        else if (argparser_keyword_present_next(pp, "down"))
          { (*vdown) = TRUE; }
        else
          { argparser_error(pp, "invalid vertical axis direction"); }
      }
  }

void imgc_parse_icenter_iorg(argparser_t *pp, bool_t *icenter, r2_t *iorg)
  {
    if (argparser_keyword_present(pp, "-icenter"))
      { (*icenter) = TRUE; }
    else if (argparser_keyword_present(pp, "-iorg"))
      { (*icenter) = FALSE;
        iorg->c[0] = argparser_get_next_double(pp, -BIG, +BIG);
        iorg->c[1] = argparser_get_next_double(pp, -BIG, +BIG);
      }
  }

void imgc_parse_ocenter_oorg(argparser_t *pp, bool_t *ocenter, r2_t *oorg)
  {
    if (argparser_keyword_present(pp, "-ocenter"))
      { (*ocenter) = TRUE; }
    else if (argparser_keyword_present(pp, "-oorg"))
      { (*ocenter) = FALSE;
        oorg->c[0] = argparser_get_next_double(pp, -BIG, +BIG);
        oorg->c[1] = argparser_get_next_double(pp, -BIG, +BIG);
      }
  }

void imgc_parse_osize(argparser_t *pp, int *ocols, int *orows, int max_size)
  {
    if (argparser_keyword_present(pp, "-osize"))
      { (*ocols) = argparser_get_next_int(pp, 1, max_size);
        (*orows) = argparser_get_next_int(pp, 1, max_size);
      }
  }
