#define PROG_NAME "test_align"
#define PROG_DESC "test of {float_image_align.h}"
#define PROG_VERS "1.0"

/* Last edited on 2008-05-25 01:58:57 by stolfi */ 
/* Created on 2007-07-11 by J. Stolfi, UNICAMP */

#define test_align_COPYRIGHT \
  "Copyright  2007  by the State University of Campinas (UNICAMP)"

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>

#include <float_image.h>
#include <float_pnm_image.h>
#include <float_image_align.h>
#include <r2.h>
#include <r2x2.h>
#include <bool.h>
#include <jsfile.h>
#include <jsrandom.h>
#include <affirm.h>
#include <assert.h>

int main(int argn, char **argv);
void do_test_align(int n, float_image_t *img[], int NC, int NX, int NY);

void choose_optimum_alignment(int n, r2_t p[], i2_t NS, r2_t step, r2_t fopt[], r2_t iopt[]);
  /* Chooses an alignment sufficiently near to the initial guess {p},
    to serve as the expected solution that will be found when calling
    {fial_optimal_shifts_brute(n, fmm, step, NS, p, &Q)}. The actual
    optimum is returned in {fopt[0..n-1]}. The best solution in the
    search grid (which differs from {fopt} by an integer number of
    steps) is returned in {iopt[0..n-1]}. Makes sure that {iopt[]} is
    quasi-balanced. */

float_image_t *read_image
  ( char *prefix, 
    int k,
    char *ext,
    int *colsP, 
    int *rowsP, 
    int *chnsP
  );
  /* Reads a PBM/PGM/PPM image file with name "{prefix}-{KK}.{EXT}", 
    where {KK} is the two-digit value of {k}, and converts it to a float
    image with samples in the range [0_1]. If {*colsP,*rowsP,*chnsP} are
    negative, sets them to the image's dimensions; otherwise requires
    that they match the image's dimensions. If {verbose} is true, 
    prints image statistics to {stderr}. */

#define NMAX 5
  /* Max number of images for simultaneous alignment. */

int main(int argn, char **argv)
  {
    srandom(4615*417);
    /* Input images for testing: */
    char *ext = "ppm"; /* Fixed for now. */
    int NX = -1; /* Number of columns. */
    int NY = -1; /* Number of rows. */
    int NC = -1; /* Number of channels. */
    float_image_t *img[NMAX];
    int k;
    for (k = 0; k < NMAX; k++) 
      { img[k] = read_image("img/in", k, ext, &NX, &NY, &NC); }
    
    /* Run test with various image counts: */
    int n;
    for (n = 0; n < NMAX; n++) 
      { do_test_align(n, img, NC, NX, NY); }
    
    return 0;
  }
  
void do_test_align(int n, float_image_t *img[], int NC, int NX, int NY)
  { 
    fprintf(stderr, "testing with %d %d-channel images...\n", n, NC);
    
    i2_t NR = (i2_t){{ 5, 7 }}; /* Radius of comparison window (pixels). */
    i2_t NS = (i2_t){{ 1, 2 }}; /* Number of alignment adjustment steps to consider. */
    r2_t step = (r2_t) {{ 0.25, 0.50 }};  /* Adjustment steps along each axis. */
    
    r2_t ini[n];  /* Initial alignment. */
    
    /* Choose the initial alignment {ini}: */
    int k, a;
    for (k = 0; k < n; k++) { ini[k] = (r2_t){{ 9, 9 }}; }
      
    /* Choose the actual optimum alignment {fopt, iopt}: */
    r2_t fopt[n]; /* Actual optimum alignment. */
    r2_t iopt[n]; /* best alignment in search space. */
    choose_optimum_alignment(n, ini, NS, step, fopt, iopt);

    auto double fmm(int n, r2_t p[]);
    /* The goal function (quadratic mismatch for alinment {p[0..n-1]}. */
    
    double Q = 0;  /* Mismatch of best alignment. */

    /* Call the optimizer: */
    r2_t sol[n];  /* Corresponding points in each image. */
    for (k = 0; k < n; k++) { sol[k] = ini[k]; }
    
    fial_debug_displacement("actual optimum  ", n, NULL, fopt, fmm(n, fopt), "\n");
    fial_debug_displacement("grid optimum    ", n, NULL, iopt, fmm(n, iopt), "\n");
    fial_debug_displacement("initial guess   ", n, NULL, sol,  fmm(n, sol), "\n");
    
    fial_optimal_shifts_brute(n, fmm, step, NS, sol, &Q);
    
    fial_debug_displacement("computed optimum", n, NULL, sol, Q, "\n");
    
    /* Check correctness of solution: */
    bool_t bug = FALSE;
    for (k = 0; k < n; k++)
      { for (a = 0; a < 2; a++)
          { if (fabs(iopt[k].c[a] - sol[k].c[a]) > 1.0e-6*step.c[a])
              { fprintf(stderr, "returned wrong solution\n");
                fprintf(stderr, " sol[%d][%d]", k, a);
                fprintf(stderr, " = %12.7f should be %12.7f\n",sol[k].c[a], iopt[k].c[a]);
                bug = TRUE;
              }
          }
      }
    if(bug) { exit(1); }
    
    return;
    
    double fmm(int n, r2_t p[])
    { /* !!! placeholder for the real thing: */
      /* Compute sum of square distances from actual optimum {opt}: */
      NR.c[0] = NR.c[0]; /* GCC pacifier. */
      double sum2 = 0;
      int k;
      for (k = 0; k < n; k++) 
        { r2_t ok = fopt[k];
          r2_t d;
          r2_sub(&(p[k]), &ok, &d);
          sum2 += r2_norm_sqr(&d);
        }
      return sum2;
    }
  }
    
void choose_optimum_alignment(int n, r2_t p[], i2_t NS, r2_t step, r2_t fopt[], r2_t iopt[])
  {
    if (n == 0) { return; }
    
    /* Min and max allowed sum of step counts from initial guess: */
    int smin = -n/2;
    int smax = smin + n - 1;

    int a;
    for (a = 0; a < 2; a++)
      { int Na = NS.c[a]; /* Max number of search steps along axis {a}. */
        double stepa = step.c[a]; /* Step size along axis {a}. */
        int da[n];        /* Step counts along axis {a}. */
        /* Set {da[0..n-1]} to random step counts: */
        int sumd = 0; /* Sum of all {da[0..n-1].c[a]}. */
        int k;
        for (k = 0; k < n; k++) 
          { /* Choose the number of steps from {p[k].c[a]} to {iopt[k].c[a]}: */
            da[k] = abrandom(-Na, +Na); /* Num of steps. */
            sumd += da[k];
          }
        /* Adjust {da} so that it is near-balanced: */
        int corr = 0;
        if (sumd > smax) { corr = -(sumd - smin)/n; }
        if (sumd < smin) { corr = (smax - sumd)/n; }
        assert((smin <= sumd + n*corr) && (sumd + n*corr <= smax)); 
        sumd = 0;
        for (k = 0; k < n; k++) { da[k] += corr; sumd += da[k]; }
        assert((smin <= sumd) && (sumd <= smax));
      
        /* Convert {da[0..n-1]} to displacements {iopt[0..n-1].c[a]} from {p}: */
        for (k = 0; k < n; k++) 
          { iopt[k].c[a] = p[k].c[a] + da[k]*stepa;
            fopt[k].c[a] = iopt[k].c[a] + 0.999999*(drandom() - 0.5)*stepa;
          }
      }
  }

float_image_t *read_image
  ( char *prefix, 
    int k,
    char *ext,
    int *colsP, 
    int *rowsP, 
    int *chnsP
  )
{ char *fname = NULL;
    asprintf(&fname, "%s-%02d.%s", prefix, k, ext);
    pnm_image_t *pim = pnm_image_read(fname);
    float_image_t *fim = float_image_from_pnm_image(pim, NULL, NULL, TRUE);
    if ((*colsP) < 0) { (*colsP) = pim->cols; } else { demand((*colsP) == pim->cols, "bad img"); } 
    if ((*rowsP) < 0) { (*rowsP) = pim->rows; } else { demand((*rowsP) == pim->rows, "bad img"); } 
    if ((*chnsP) < 0) { (*chnsP) = pim->chns; } else { demand((*chnsP) == pim->chns, "bad img"); } 
    pnm_image_free(pim);
    free(fname);
    return fim;
  }

