/* See double_spmat.h */
/* Last edited on 2008-05-25 03:24:31 by stolfi */

#include <double_spmat.h>

#include <affirm.h>
#include <fget.h>
#include <nget.h>
#include <jsfile.h>

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

/* IMPLEMENTATIONS */

vec_typeimpl(double_spmat_t, double_spmat, double_spmat_entry_t);

double_spmat_t double_spmat_read(char *name)
  { FILE *rd = open_read(name, TRUE);
    double_spmat_t M = double_spmat_fread(rd);
    if (rd != stdin) { fclose(rd); }
    return M;
  }
  
double_spmat_t double_spmat_fread(FILE *rd)
  { int nel = nget_int(rd, "elems"); fget_eol(rd);
    double_spmat_t M = double_spmat_new(nel);
    int i;
    for (i = 0; i < nel; i++) 
      { double_spmat_entry_t *e = &(M.el[i]);
        e->row = fget_int(rd);
        e->col = fget_int(rd);
        e->val = fget_double(rd);
        fget_eol(rd);
      }
    return M;
  }
  
void double_spmat_write(char *name, double_spmat_t M)
  { FILE *wr = open_write(name, TRUE);
    double_spmat_fwrite(wr, M);
    if (wr == stdout) { fflush(wr); } else { fclose(wr); }
  }

void double_spmat_fwrite(FILE *wr, double_spmat_t M)
  { int i;
    fprintf(wr, "elems = %d\n", M.nel);
    for (i = 0; i < M.nel; i++) 
      { double_spmat_entry_t *e = &(M.el[i]);
        fprintf(wr, "%5d %5d %24.16e\n", e->row, e->col, e->val); }
    fflush(wr);
  }

void double_spmat_sort_entries
  ( double_spmat_entry_t e[],
    int jini,
    int jlim,
    double_spmat_entry_cmp_func_t *cmp
  )
  { 
    int j;
    for (j = jini + 1; j < jlim; j++)
      { int k = j, p = k - 1;
        double_spmat_entry_t *ek = &(e[k]);
        double_spmat_entry_t *ep = &(e[p]);
        if (cmp(ep, ek) > 0)
          { /* Entry {*ek} is out of order, insert it in its proper place: */
            double_spmat_entry_t etmp = (*ek);
            do
              { (*ek) = (*ep);
                k = p; ek = ep;
                if (k <= jini) { break; }
                p--; ep--;
              } 
            while (cmp(ep, &etmp) > 0);
            (*ek) = etmp; 
          }
      }
  }
