/* double_spmat - tools for sparse matrices of {double}s */
/* Last edited on 2008-02-23 08:11:14 by stolfi */

#ifndef double_spmat_H
#define double_spmat_H

#include <bool.h>
#include <vec.h>
#include <sign.h>
#include <stdio.h>

/* !!! Should be generalized for arbitrary element types, like {vec.h}. */

typedef struct double_spmat_entry_t 
  { int row;     /* Row index. */
    int col;     /* Column index. */
    double val;  /* Element value. */
  } double_spmat_entry_t;
  /* An entry in a sparse matrix. */
  
vec_typedef(double_spmat_t, double_spmat, double_spmat_entry_t);
  /* A {double_spmat_t} is a {vec} of {double_spmat_entry_t} items, in some order. */

void double_spmat_write(char *name, double_spmat_t M);
  /* Writes to file {name} the matrix {M}.
    If {name} is "-", writes to {stdout}. */
  
void double_spmat_fwrite(FILE *wr, double_spmat_t M);
  /* Same as {double_spmat_write}, but writes to the client-opened file {wr}. */

double_spmat_t double_spmat_read(char *name);
  /* Reads from file {name} a matrix {M}. The file must be in the
    format created by {double_spmat_write}. If {name} is "-", reads
    from {stdin}. */

double_spmat_t double_spmat_fread(FILE *rd);
  /* Same as {double_spmat_read}, but reads from the client-opened file {rd}. */

typedef sign_t double_spmat_entry_cmp_func_t(double_spmat_entry_t *ea, double_spmat_entry_t *eb);
  /* Type of a procedure that compares two enries {*ea} and {*eb}, and returns
    {-1} if {*ea} should come first, {+1} if {*eb} should come first, 0 if they 
    are equivalent. */

void double_spmat_sort_entries
  ( double_spmat_entry_t e[],
    int jini,
    int jlim,
    double_spmat_entry_cmp_func_t *cmp
  ); 
  /* Rearranges the entries {e[jini..jlim-1]} so that they are in
    increasing order according to the partial order {cmp}.
    Uses insertion sort, assuming that the entries are
    already mostly sorted. */

#endif
