#define PROG_NAME "testjsmath"
#define PROG_DESC "test of {jsmath.h}"
#define PROG_VERS "1.0"

/* Last edited on 2008-01-16 16:23:51 by stolfi */ 
/* Created on 2007-01-02 by J. Stolfi, UNICAMP */

#define testjsmath_COPYRIGHT \
  "Copyright  2007  by the State University of Campinas (UNICAMP)"

#include <jsmath.h>
#include <jsrandom.h>
#include <affirm.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#define bug(FMT_AND_ARGS...) \
  do { \
    fprintf(stderr, "%s:%d: ", __FILE__, __LINE__); \
    fprintf(stderr, FMT_AND_ARGS); \
    fprintf(stderr, "\n"); \
    exit(1); \
  } while(0)

uint64_t uint64_random();
int64_t int64_random();

void test_gcd(int nt);
void test_lcm(int nt);
void test_imod(int nt);
/* !!! should test ifloor, iceil */
void test_ipow(int nt);

#define N_uint64_nice 14

static uint64_t uint64_nice[N_uint64_nice] = 
  { 1, 
    2, 
    3, 
    4, 
    12, 
    1023, 
    1024, 
    1025, 
    4294967295ULL, /* 2^32-1 */
    4294967296ULL, /* 2^32 */
    4294967297ULL, /* 2^32+1 */
    18446744073709551614ULL, /* 2^64-2 */
    18446744073709551615ULL, /* 2^64-1 */
    0
  };

#define N_int64_nice 28

static int64_t int64_nice[N_int64_nice] = 
  { +1,                       -1, 
    +2,                       -2, 
    +3,                       -3, 
    +4,                       -4, 
    +12,                      -12,
    +1023,                    -1023,                  
    +1024,                    -1024,                  
    +1025,                    -1025,                  
    +4294967295LL,            -4294967295LL,             /* 2^32-1 */
    +4294967296LL,            -4294967296LL,             /* 2^32 */
    +4294967297LL,            -4294967297LL,             /* 2^32+1 */
    +9223372036854775806LL,   -9223372036854775806LL,    /* 2^63-2 */
    +9223372036854775807LL,   -9223372036854775807LL,    /* 2^63-1 */
    ((int64_t)-9223372036854775807LL)-1,                 /* -2^63 */
    0
  };

int main (int argn, char **argv)
  { test_gcd(200);
    test_lcm(200);
    test_imod(200);
    test_ipow(200);

    return 0;
  }

void test_gcd(int nt)
  { fprintf(stderr, "Checking {gcd}...\n");
    int i,j;
    for (i = 0; i < N_uint64_nice + nt; i++)
      { uint64_t a = (i < N_uint64_nice ? uint64_nice[i] : uint64_random());
        for (j = 0; j < N_uint64_nice + nt; j++)
          { uint64_t b = (j < N_uint64_nice ? uint64_nice[j] : uint64_random());
            uint64_t c = gcd(a, b);
            if (c == 0)
              { affirm((a == 0) && (b == 0), "zero gcd"); }
            else
              { affirm(a % c == 0, "gcd does not divide a");
                affirm(b % c == 0, "gcd does not divide b");
                affirm(gcd(a/c, b/c) == 1, "gcd(a/c,b/c) is not 1");
                /* Should check maximality... */
              }
          }
      }
  }

void test_lcm(int nt)
  { fprintf(stderr, "Checking {lcm}...\n");
    int i,j;
    for (i = 0; i < N_uint64_nice + nt; i++)
      { uint64_t a = (i < N_uint64_nice ? uint64_nice[i] : uint64_random());
        for (j = 0; j < N_uint64_nice + nt; j++)
          { uint64_t b = (j < N_uint64_nice ? uint64_nice[j] : uint64_random());
            uint64_t g = gcd(a, b);
            if ((a == 0) || (b == 0))
              { affirm(lcm(a,b) == 0, "lcm of zero is not zero"); }
            else if (a/g <= UINT64_MAX/b)
              { int64_t c = lcm(a, b);
                affirm(c != 0, "zero lcm");
                affirm(c % a == 0, "lcm does not divide a");
                affirm(c % b == 0, "lcm does not divide b");
                /* Should check minimality... */
              }
          }
      }
  }

void test_imod(int nt)
  { fprintf(stderr, "Checking {imod}...\n");
    int i,j;
    for (i = 0; i < N_int64_nice + nt; i++)
      { int64_t a = (i < N_int64_nice ? int64_nice[i] : int64_random());
        for (j = 0; j < N_int64_nice + nt; j++)
          { int64_t b = (j < N_int64_nice ? int64_nice[j] : int64_random());
            if (b > 0)
              { int64_t c = imod(a, b);
                affirm(c >= 0, "imod is negative");
                affirm(c < b, "imod is too big");
                /* Check {(a - c)%b}, but beware of overflow: */
                if (a < 0)
                  { affirm((a + (b - c)) % b == 0, "imod is not remainder"); }
                else
                  { affirm((a - c) % b == 0, "imod is not remainder"); }
                  
                /* Should check minimality... */
              }
          }
      }
  }

void test_ipow(int nt)
  { fprintf(stderr, "Checking {ipow}...\n");
    fprintf(stderr, "*** test not implemented yet! ***\n");
  }
